# Copyright 2014 Deezer (http://www.deezer.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Replaces the template markup (PHP, Jinja, Mako) from an HTML file.

It replaces the markup so the lines and positions of actual HTML content is
preserved.

It uses Regexes to do the replacements so it is prone to errors in some corner
cases.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from io import StringIO

import re


# Common patterns
LEADING_SPACES = r'(?P<spaces>^[\t ]+)'
NEW_LINE = r'(?P<newline>\n\r|\n|\r\n)'

# PHP patterns
PHP_START_ECHO = r'<\?='
PHP_START_TAG_WITH_ECHO = r'<\?php\s+echo'
PHP_START_TAG_SHORT_WITH_ECHO = r'<\?\s+echo'
PHP_START_TAG = r'<\?php'
# N.B.(skreft): there's no need to use a negative lookahead here [(?!=)], as the
# priority of start tags is lower than the one of echo tags.
PHP_START_TAG_SHORT = r'<\?'
PHP_END_TAG = r'\?>'

# Jinja patterns
JINJA_START_TAG = r'{%'
JINJA_START_ECHO = r'{{'
JINJA_END_TAG = r'%}'
JINJA_END_ECHO = r'}}'

# Mako patterns
MAKO_START_ECHO = r'\${'
MAKO_END_ECHO = r'}'
MAKO_START_TAG = (
    r'(</?%|%\s*(end)?(if|else|elif|for|while|try|catch|except|finally)|##)')
MAKO_END_TAG = r'([%/]?>|$)'


def get_pattern(echo_tags, start_tags, end_tags):
    """Constructs a pattern to use with the method clean().

    The priority of the tags is echo_tags, then start_tags and finally end_tags.
    That means that in some cases a negative lookahead may be required, as the
    first match from left to right will be reported.

    See clean() for the meaning of each group.

    Args:
      echo_tags: list of re strings. These are tags that may output something.
      start_tags: list of re strings. Tags that typically do not output
        anything, like control structures.
      end_tags: list of re strings. All the closing tags.

    Returns:
      a RegexObject.
    """
    return re.compile(
        '|'.join((
            LEADING_SPACES,
            NEW_LINE,
            r'(?P<echo>%s)' % '|'.join(echo_tags),
            r'(?P<start>%s)' % '|'.join(start_tags),
            r'(?P<end>%s)' % '|'.join(end_tags))),
        flags=re.MULTILINE)


JINJA_PATTERN = get_pattern(
    [JINJA_START_ECHO],
    [JINJA_START_TAG],
    [JINJA_END_TAG, JINJA_END_ECHO])

PHP_PATTERN = get_pattern(
    [PHP_START_ECHO, PHP_START_TAG_WITH_ECHO, PHP_START_TAG_SHORT_WITH_ECHO],
    [PHP_START_TAG, PHP_START_TAG_SHORT],
    [PHP_END_TAG])

MAKO_PATTERN = get_pattern(
    [MAKO_START_ECHO],
    [MAKO_START_TAG],
    [MAKO_END_TAG, MAKO_END_ECHO])

ALL_PATTERN = get_pattern(
    [PHP_START_ECHO, PHP_START_TAG_WITH_ECHO, PHP_START_TAG_SHORT_WITH_ECHO,
        JINJA_START_ECHO, MAKO_START_ECHO],
    [PHP_START_TAG, PHP_START_TAG_SHORT, JINJA_START_TAG, MAKO_START_TAG],
    [PHP_END_TAG, JINJA_END_TAG, JINJA_END_ECHO, MAKO_END_TAG, MAKO_END_ECHO])


def _get_tag(match):
    groups = match.groupdict()
    if groups.get('echo') is not None:
        return 'ECHO'
    elif groups.get('end') is not None:
        return 'END'
    elif groups.get('start') is not None:
        return 'START'
    elif groups.get('newline') is not None:
        return 'NEWLINE'
    elif groups.get('spaces') is not None:
        return 'SPACES'

    print(groups)
    assert False, ('Only the groups "echo", "end", "start", "newline" and ' +
                   '"spaces" are allowed. Please correct your pattern or use ' +
                   'the method get_pattern() to construct it.')


class _TemplateRemover(object):
    """Helper class for the clean() method.

    This class exists mainly to factor out some methods.
    """
    def __init__(self, html_content, pattern=ALL_PATTERN):
        self.html_content = html_content
        self.pattern = pattern
        self._output = StringIO()
        self._index = 0
        self._state = 'HTML'
        self._pending = []
        self._pending_has_blank = False

    def _reset_pending(self):
        self._pending = []
        self._pending_has_blank = False

    def _write_content(self, end=None):
        self._output.writelines(self._pending)
        self._reset_pending()
        self._output.write(self.html_content[self._index:end])

    def get_clean_content(self):
        """Implementation of the clean() method."""
        fill_chars = {'BLANK_TEMPLATE': ' ', 'ECHO_TEMPLATE': '0'}
        for match in self.pattern.finditer(self.html_content):
            start, end = match.start(), match.end()
            tag = _get_tag(match)
            if tag == 'ECHO':
                self._write_content(start)
                self._index = start
                self._state = 'ECHO_TEMPLATE'
            elif tag == 'START':
                if self._index != start:
                    self._write_content(start)
                self._index = start
                self._state = 'BLANK_TEMPLATE'
            elif tag == 'END':
                if self._state not in ('BLANK_TEMPLATE', 'ECHO_TEMPLATE'):
                    # We got a closing tag but none was open. We decide to carry
                    # on as it may be the case that it was because of a closing
                    # dictionary in javascript like: var dict = {foo:{}}.
                    # See the note on the clean() function for more details.
                    continue
                fill_char = fill_chars[self._state]
                fill = fill_char * (end - self._index)
                if self._state == 'BLANK_TEMPLATE':
                    self._pending.append(fill)
                    self._pending_has_blank = True
                else:
                    assert not self._pending
                    self._output.write(fill)
                self._index = end
                self._state = 'HTML'
            elif tag == 'SPACES':
                self._pending.append(match.group('spaces'))
                self._index = end
            elif tag == 'NEWLINE':
                if self._state == 'HTML':
                    if self._index != start or not self._pending_has_blank:
                        self._write_content(start)
                    self._output.write(match.group('newline'))
                elif self._state == 'BLANK_TEMPLATE':
                    # We discard the content of this template and whatever is in
                    # self._pending.
                    self._output.write(match.group('newline'))
                elif self._state == 'ECHO_TEMPLATE':
                    assert False, 'Echo tags should be in just one line.'
                self._index = end
                self._reset_pending()

        assert self._state == 'HTML', 'Tag was not closed'
        if self._index != len(self.html_content) or not self._pending_has_blank:
            self._write_content()

        return self._output.getvalue()


def clean(html_content, pattern=ALL_PATTERN):
    """Removes the markup from the supplied string.

    Note: this is not a fully compliant markup remover as it is only based in
    some regular expressions. This means there are some edge cases that cannot
    be captured with this method. Although we believe those cases are too
    contrived, and probably should be avoided as some development tools will
    fail as well.

    One example that won't work is the following:
        <?php echo "?>" ?>
    The reason it does not work is because when the method sees the first '?>'
    (the one inside the string), it thinks it's a closing tag.

    This method works by finding the beginning and ending of the tags. The tags
    are grouped in three categories.
      echo_tags: these are tags that will typically produce some output and
        should be replaced with actual content not blanks. It will output as
        many zeroes as the length of the original tag. The length is preserved
        so the columns and lines are preserved for the html. The filling
        character is a '0', just because is a valid value for a number and some
        html attributes require numbers.
        Strong assumption: this tag should be in just one line.
        For example, href="<?= $foo ?>" is replaced with href="00000000000".
      start_tags: these tags do not usually print content, these are mainly
        control structures. The content of this tag is replaced by blanks or
        removed depending on the context. If the tag is open and closed in the
        same line then it is replaced by the same number of blanks, but only if
        there was nonblank content before or after. If there is multiline then
        the contents are just removed. The contents are stripped, so there are
        no trailing spaces introduced by the markup.
        For example:
        "  <?php if { ?>" is replaced by ""
        but
        "  <?php if { ?>  <!-- A comment ->" is replaced by
        "                 <!-- A comment ->"
      end_tags: the closing tags for both echo and start tags. They trigger the
      output.

    Args:
      html_content: the string to be cleaned.
      pattern: a RegexObject containing only the groups "echo", "end", "start",
        "newline" and "spaces". Use the method get_pattern to construct it.
        By default it will clean all defined markups (PHP and Jinja).

    Returns:
      A string with the markup removed.
    """
    return _TemplateRemover(html_content, pattern).get_clean_content()


def clean_php(html_content):
    """Removes the PHP markup from the supplied string.

    It does not support the optional asp tags.

    See clean() for more details.
    """
    return clean(html_content, pattern=PHP_PATTERN)


def clean_jinja(html_content):
    """Removes the Jinja markup from the supplied string.

    See clean() for more details.
    """
    return clean(html_content, pattern=JINJA_PATTERN)


def clean_mako(html_content):
    """Removes the Mako markup from the supplied string.

    See clean() for more details.
    """
    return clean(html_content, pattern=MAKO_PATTERN)



#!/usr/bin/env python
# Copyright 2014 Deezer (http://www.deezer.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""remove_template.py

Removes the PHP or Jinja2 markup from html files.

This software is released under the Apache License. Copyright Deezer 2014.

Usage:
  remove_template.py FILENAME
  remove_template.py (-h | --help)
  remove_template.py --version

Options:
  -h --help        Show this screen.
  --version        Show version.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import codecs
import io
import sys

import docopt

import template_remover


__VERSION__ = '0.1'


def main():
    """Entry point for remove_template."""

    # Wrap sys stdout for python 2, so print can understand unicode.
    if sys.version_info[0] < 3:
        sys.stdout = codecs.getwriter("utf-8")(sys.stdout)

    options = docopt.docopt(__doc__,
                            help=True,
                            version='template_remover v%s' % __VERSION__)

    print(template_remover.clean(io.open(options['FILENAME']).read()))

    return 0

if __name__ == '__main__':
    sys.exit(main())



# Copyright 2014 Deezer (http://www.deezer.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the template_remover module."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import re
import unittest

import template_remover

# pylint: disable=too-many-public-methods


class TestTemplateRemover(unittest.TestCase):
    def test_clean_invalid_pattern(self):
        with self.assertRaises(AssertionError):
            template_remover.clean('foo', re.compile(r'\w'))

    def test_clean_php_open_tag_at_eof(self):
        with self.assertRaises(AssertionError):
            template_remover.clean_php('<?php')

    def test_clean_php_inline_echo(self):
        self.assertEquals(
            '<a href="00000000000">Link</a>',
            template_remover.clean_php('<a href="<?= $foo ?>">Link</a>')
        )

    def test_clean_php_inline_echho_not_closed(self):
        with self.assertRaises(AssertionError):
            template_remover.clean_php('<a href="<?= $foo \n')

    def test_clean_php_echo(self):
        self.assertEquals(
            '<a href="0000000000000000000">Link</a>',
            template_remover.clean_php('<a href="<?php echo $foo; ?>">Link</a>')
        )

    def test_clean_php_short_echo(self):
        self.assertEquals(
            '<a href="0000000000000000">Link</a>',
            template_remover.clean_php('<a href="<? echo $foo; ?>">Link</a>')
        )

    def test_clean_php_inline_echo_mixed(self):
        self.assertEquals(
            '         <a href="00000000000">Link</a>',
            template_remover.clean_php(
                ' <?php ?><a href="<?= $foo ?>">Link</a>')
        )

    def test_clean_php_control(self):
        self.assertEquals(
            '                      <a href="#next">Next</a>',
            template_remover.clean_php(
                '<?php if ($a > 1) { ?><a href="#next">Next</a><?php } ?>')
        )

    def test_clean_php_short_tags(self):
        self.assertEquals(
            'a       b',
            template_remover.clean_php('a <? ?> b')
        )

    def test_clean_php_control_multiline(self):
        self.assertEquals(
            '\n\n',
            template_remover.clean_php(
                '  <?php if ($a > 1) {    \n echo "foo"\n  } ?>')
        )

    def test_clean_php_multiline(self):
        self.assertEquals(
            '                      <a href="#next">Next</a>\n' +
            '<a href="00000000000">Link</a>',
            template_remover.clean_php(
                '<?php if ($a > 1) { ?><a href="#next">Next</a><?php } ?>\n' +
                '<a href="<?= $foo ?>">Link</a>')
        )

    def test_spaces_are_preserved(self):
        self.assertEquals(
            '  \t\n    ',
            template_remover.clean_php('  \t\n    ')
        )

    def test_php_control_at_end(self):
        self.assertEquals(
            '            placeholder  ',
            template_remover.clean_php('  <?php ?>  placeholder  ')
        )

    def test_unicode(self):
        self.assertEquals(
            'ni\xf1o',
            template_remover.clean_php('ni\xf1o')
        )
        self.assertEquals(
            '\xe9',
            template_remover.clean_php('\xe9')
        )

    def test_php_comprehensive(self):
        template = '''<html>
          <body>
          
          <?php if($foo) { ?>
            <a href="<?= $foo ?>"><a>
          <?php } ?>  <!-- comment ->

          
        '''
        expected = '''<html>
          <body>
          

            <a href="00000000000"><a>
                      <!-- comment ->

          
        '''
        self.assertEquals(expected, template_remover.clean_php(template))

    def test_jinja_comprehensive(self):
        template = '''<html>
          <body>
          
          {% if foo %}
            <a href="{{foo}}"><a>
          {% endif %}  <!-- comment ->

          
        '''
        expected = '''<html>
          <body>
          

            <a href="0000000"><a>
                       <!-- comment ->

          
        '''
        self.assertEquals(expected, template_remover.clean_jinja(template))

    def test_jinja_does_not_complain_with_double_braces(self):
        template = '<script>var dict={foo:{}}</script>'
        expected = '<script>var dict={foo:{}}</script>'
        self.assertEquals(expected, template_remover.clean_jinja(template))

    def test_mako_comprehensive(self):
        template = '''<html>
          <body>
          
          <a title="${"this is some text" | u}">Foo</a>
          % if x == 5:
            <a href="${foo}"><a>
          % endif
          <%
            x = db.get_resource('foo')
            y = [z.element for z in x if x.frobnizzle==5]
          %>
          <%include file="foo.txt"/>
          ## Comment
          <%def name="foo" buffered="True">
            this is a def
          </%def>
          
        '''
        expected = '''<html>
          <body>
          
          <a title="00000000000000000000000000">Foo</a>

            <a href="000000"><a>








            this is a def

          
        '''
        self.assertEquals(expected, template_remover.clean_mako(template))



# Copyright 2014 Deezer (http://www.deezer.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages


setup(
    name='template-remover',
    version='0.1.7',
    description='Remove the template markup from html files',
    long_description=open('README.rst').read(),
    author='Sebastian Kreft - Deezer',
    author_email='skreft@deezer.com',
    url='http://github.com/deezer/template-remover',
    py_modules=['template_remover'],
    install_requires=['docopt==0.6.1'],
    tests_require=['nose>=1.3'],
    scripts=['scripts/remove_template.py'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development',
    ],
)
